@@ -23,6 +23,7 @@ import contextlib
 import logging
 import os
 import shutil
+import itertools
 
 from ubuntuone.oauthdesktop.main import Login, LoginProcessor
 from ubuntuone.syncdaemon import (
@@ -508,7 +509,7 @@ class FakeVolumeManager(object):
 
     def __init__(self, root_path):
         """ Creates the instance"""
-        self.root = volume_manager.Share(path=root_path, access_level='Modify')
+        self.root = volume_manager.Root(node_id="root_node_id", path=root_path)
         self.shares = {'':self.root}
         self.udfs = {}
         self.log = logging.getLogger('ubuntuone.SyncDaemon.VM-test')
@@ -540,14 +541,12 @@ class FakeVolumeManager(object):
         except KeyError:
             return self.udfs[id]
 
-    def get_volumes(self, access_level='Modify', all_volumes=False):
+    def get_volumes(self, all_volumes=False):
         """Simple get_volumes for FakeVolumeManager."""
-        for share in self.shares.values():
-            if all_volumes or share.access_level == access_level:
-                yield share
-        for udf in self.udfs.values():
-            if all_volumes or udf.subscribed:
-                yield udf
+        volumes = itertools.chain(self.shares.values(), self.udfs.values())
+        for volume in volumes:
+            if all_volumes or volume.active:
+                yield volume
 
     def unsubscribe_udf(self, udf_id):
         """Mark the UDF with udf_id as unsubscribed."""
