@@ -1033,8 +1033,7 @@ class FilterEventsTestCase(BasicTestCase):
     def test_SV_HASH_NEW_is_pushed_for_subscrined_volume(self):
         """SV_HASH_NEW is filtered when the volume is unsubscribed."""
         udf_id = 'udf_id'
-        udf_volume = volumes.UDFVolume(udf_id, 'udf_node',
-                                       u'~/ñoño'.encode("utf8"))
+        udf_volume = volumes.UDFVolume(udf_id, 'udf_node', u'~/ñoño')
         path = self.vm._build_udf_path(udf_volume.suggested_path)
         udf = UDF.from_udf_volume(udf_volume, path)
         yield self.vm.add_udf(udf)
@@ -1048,12 +1047,11 @@ class FilterEventsTestCase(BasicTestCase):
                            self.action_queue.event_queue.events)
 
     @defer.inlineCallbacks
-    def test_SV_HASH_NEW_is_filtered_for_unsubscrined_volume(self):
+    def test_SV_HASH_NEW_is_filtered_for_unsubscribed_volume(self):
         """SV_HASH_NEW is filtered when the volume is unsubscribed."""
         # build a VM and add it an UDF with subscribed to False
         udf_id = 'udf_id'
-        udf_volume = volumes.UDFVolume(udf_id, 'udf_node',
-                                       u'~/ñoño'.encode("utf8"))
+        udf_volume = volumes.UDFVolume(udf_id, 'udf_node', u'~/ñoño')
         path = self.vm._build_udf_path(udf_volume.suggested_path)
         udf = UDF.from_udf_volume(udf_volume, path)
         yield self.vm.add_udf(udf)
