@@ -37,7 +37,12 @@ from ubuntuone.syncdaemon.dbus_interface import (
     DBUS_IFACE_PUBLIC_FILES_NAME,
     EventListener,
 )
-from ubuntuone.syncdaemon.volume_manager import Share, Shared, UDF
+from ubuntuone.syncdaemon.volume_manager import (
+    Share,
+    Shared,
+    UDF,
+    VolumeDoesNotExist,
+)
 from ubuntuone.syncdaemon.tools import DBusClient
 from ubuntuone.syncdaemon import event_queue, states, main, config
 from contrib.testing.testcase import (
@@ -799,7 +804,7 @@ class DBusInterfaceTests(DBusTwistedTestCase):
         expected = []
         udf_id = str(uuid.uuid4())
         udf_path = self.main.vm._build_udf_path('~/foo/bar')
-        udf = UDF(str(udf_id), str('udf_node_id'), '~/foo/bar', udf_path, True)
+        udf = UDF(str(udf_id), str('udf_node_id'), u'~/foo/bar', udf_path, True)
         self.main.vm.udfs[udf_id] = udf
         for i in xrange(5):
             if i % 2:
@@ -1821,6 +1826,9 @@ class FolderTests(DBusTwistedTestCase):
     def _create_udf(self, id, node_id, suggested_path, subscribed=True):
         """Create an UDF and returns it and the volume"""
         path = self.main.vm._build_udf_path(suggested_path)
+        # make sure suggested_path is unicode
+        if isinstance(suggested_path, str):
+            suggested_path = suggested_path.decode('utf-8')
         udf = UDF(str(id), str(node_id), suggested_path, path, subscribed)
         return udf
 
@@ -1837,6 +1845,21 @@ class FolderTests(DBusTwistedTestCase):
         self.assertEquals(udf_dict['node_id'], udf.node_id)
         self.assertFalse(udf_dict['subscribed'])
 
+    def test_get_udf_dict_bad_encoding(self):
+        """Test for Folders._get_udf_dict."""
+        suggested_path = u'~/Música'
+        udf = self._create_udf(uuid.uuid4(), 'node_id', suggested_path,
+                               subscribed=False)
+        udf.suggested_path = udf.suggested_path.encode('utf-8')
+        udf_dict = self.dbus_iface.folders._get_udf_dict(udf)
+        # check the path it's unicode
+        self.assertEquals(udf_dict['path'], udf.path.decode('utf-8'))
+        self.assertEquals(udf_dict['volume_id'], udf.id)
+        self.assertEquals(repr(udf_dict['suggested_path']),
+                          repr(udf.suggested_path.decode('utf-8')))
+        self.assertEquals(udf_dict['node_id'], udf.node_id)
+        self.assertFalse(udf_dict['subscribed'])
+
     @defer.inlineCallbacks
     def test_get_folders(self):
         """Test for Folders.get_folders."""
@@ -2018,7 +2041,7 @@ class FolderTests(DBusTwistedTestCase):
             """FolderDeleted handler."""
             self.assertRaises(KeyError, self.main.fs.get_by_path,
                               info['path'].decode('utf-8'))
-            self.assertRaises(KeyError, self.main.vm.get_volume, info['volume_id'])
+            self.assertRaises(VolumeDoesNotExist, self.main.vm.get_volume, info['volume_id'])
             d.callback(True)
         match = self.bus.add_signal_receiver(deleted_handler,
                                              signal_name='FolderDeleted')
@@ -2027,7 +2050,7 @@ class FolderTests(DBusTwistedTestCase):
             """the callback"""
             self.assertNotIn(udf.volume_id, self.main.vm.udfs)
             self.assertRaises(KeyError, self.main.fs.get_by_path, udf.path)
-            self.assertRaises(KeyError, self.main.vm.get_volume, udf.volume_id)
+            self.assertRaises(VolumeDoesNotExist, self.main.vm.get_volume, udf.volume_id)
         self.folders_client.call_method('delete', udf.volume_id,
                                         reply_handler=check_deleted,
                                         error_handler=self.error_handler)
@@ -2059,6 +2082,24 @@ class FolderTests(DBusTwistedTestCase):
         d.addCallback(lambda _: self.main.vm.udf_deleted(udf.volume_id))
         return d
 
+    @defer.inlineCallbacks
+    def test_delete_error_signal_folder_id(self):
+        """Test for FolderDeleteError for a volume that doesn't exists."""
+        udf_id = 'foobar'
+        d = defer.Deferred()
+        def deleted_error_handler(info, error):
+            """FolderDeleteError handler"""
+            d.callback((info, error))
+        match = self.bus.add_signal_receiver(deleted_error_handler,
+                                             signal_name='FolderDeleteError')
+        self.signal_receivers.add(match)
+        self.folders_client.call_method('delete', udf_id,
+                                        reply_handler=lambda *args: None,
+                                        error_handler=d.errback)
+        info, error = yield d
+        self.assertEquals(info['volume_id'], udf_id)
+        self.assertEquals(error, "DOES_NOT_EXIST")
+
     @defer.inlineCallbacks
     def test_subscribe(self):
         """Test for Folders.subscribe and that it fires a dbus signal."""
@@ -2164,7 +2205,7 @@ class ShareTests(DBusTwistedTestCase):
             """FolderDeleted handler."""
             self.assertRaises(KeyError, self.main.fs.get_by_path,
                               info['path'].decode('utf-8'))
-            self.assertRaises(KeyError, self.main.vm.get_volume, info['volume_id'])
+            self.assertRaises(VolumeDoesNotExist, self.main.vm.get_volume, info['volume_id'])
             d.callback(True)
         match = self.bus.add_signal_receiver(deleted_handler,
                                              signal_name='ShareDeleted')
@@ -2173,7 +2214,7 @@ class ShareTests(DBusTwistedTestCase):
             """the callback"""
             self.assertNotIn(share.volume_id, self.main.vm.shares)
             self.assertRaises(KeyError, self.main.fs.get_by_path, share.path)
-            self.assertRaises(KeyError, self.main.vm.get_volume, share.volume_id)
+            self.assertRaises(VolumeDoesNotExist, self.main.vm.get_volume, share.volume_id)
         client = DBusClient(self.bus, '/shares', DBUS_IFACE_SHARES_NAME)
         client.call_method('delete_share', share.volume_id,
                            reply_handler=check_deleted,
