@@ -49,7 +49,8 @@ class WatchTests(BaseEQTestCase):
     def _create_udf(self, path):
         """Create an UDF and returns it and the volume"""
         os.makedirs(path)
-        udf = volume_manager.UDF("vol_id", "node_id", path, path, True)
+        udf = volume_manager.UDF("vol_id", "node_id", path.decode('utf-8'),
+                                 path, True)
         self.vm.add_udf(udf)
 
     def test_add_general_watch(self):
@@ -221,6 +222,12 @@ class BaseTwisted(BaseEQTestCase):
     def setUp(self):
         """Setup the test."""
         BaseEQTestCase.setUp(self)
+
+        # add a Memento handler to the logger
+        self.handler = testcase.MementoHandler()
+        self.handler.setLevel(logging.INFO)
+        self.eq.log.addHandler(self.handler)
+
         # create the deferred for the tests
         self._deferred = defer.Deferred()
 
@@ -366,8 +373,17 @@ class SignalingTests(BaseTwisted):
             def handle_FS_FILE_DELETE(innerself, path):
                 if path != testfile:
                     self.finished_error("received a wrong path")
-                else:
-                    self.finished_ok()
+                    return
+
+                check = len(self.handler.records) == 1 and \
+                        'FS_FILE_DELETE' in self.handler.records[0].message and\
+                        testfile in self.handler.records[0].message and \
+                        self.handler.records[0].levelno == logging.INFO
+                if not check:
+                    self.finished_error("FS_FILE_DELETE must appear in INFO.")
+                    return
+
+                self.finished_ok()
 
         self.eq.inotify_add_watch(self.root_dir)
         self.eq.subscribe(HitMe())
@@ -387,8 +403,17 @@ class SignalingTests(BaseTwisted):
             def handle_FS_DIR_DELETE(innerself, path):
                 if path != testdir:
                     self.finished_error("received a wrong path")
-                else:
-                    self.finished_ok()
+                    return
+
+                check = len(self.handler.records) == 1 and \
+                        'FS_DIR_DELETE' in self.handler.records[0].message and\
+                        testdir in self.handler.records[0].message and \
+                        self.handler.records[0].levelno == logging.INFO
+                if not check:
+                    self.finished_error("FS_DIR_DELETE must appear in INFO.")
+                    return
+
+                self.finished_ok()
 
         self.eq.inotify_add_watch(self.root_dir)
         self.eq.subscribe(HitMe())
