@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-'''Tests for the Event Queue.'''
+"""Tests for the Event Queue."""
 
 import logging
 import unittest
@@ -31,17 +31,19 @@ from twisted.internet import defer
 
 
 class BaseEQTestCase(testcase.BaseTwistedTestCase):
-    ''' Setup an EQ for test. '''
+    """ Setup an EQ for test. """
 
     def setUp(self):
-        '''Setup the test.'''
+        """Setup the test."""
         testcase.BaseTwistedTestCase.setUp(self)
         self.fsmdir = self.mktemp('fsmdir')
         self.partials_dir = self.mktemp('partials_dir')
         self.root_dir = self.mktemp('root_dir')
+        self.home_dir = self.mktemp('home_dir')
+        self.vm = testcase.FakeVolumeManager(self.root_dir)
         self.fs = filesystem_manager.FileSystemManager(self.fsmdir,
                                                        self.partials_dir,
-                                   testcase.FakeVolumeManager(self.root_dir))
+                                                       self.vm)
         self.fs.create(path=self.root_dir,
                        share_id='', is_dir=True)
         self.fs.set_by_path(path=self.root_dir,
@@ -50,16 +52,19 @@ class BaseEQTestCase(testcase.BaseTwistedTestCase):
         self.fs.register_eq(self.eq)
 
     def tearDown(self):
-        '''Clean up the tests.'''
-        testcase.BaseTwistedTestCase.tearDown(self)
+        """Clean up the tests."""
+        for listener in self.eq._listeners:
+            self.eq.unsubscribe(listener)
         self.eq.shutdown()
+        self.rmtree(self.tmpdir)
+        testcase.BaseTwistedTestCase.tearDown(self)
 
 
 class SubscriptionTests(BaseEQTestCase):
-    '''Test to subscribe and unsubscribe to the EQ.'''
+    """Test to subscribe and unsubscribe to the EQ."""
 
     def test_subscription(self):
-        '''Test that subscription works.'''
+        """Test that subscription works."""
         # pylint: disable-msg=W0212
         # simple one
         o = object()
@@ -73,7 +78,7 @@ class SubscriptionTests(BaseEQTestCase):
         self.assertTrue(self.eq._listeners.count(o) == 1)
 
     def test_unsubscription(self):
-        '''Test that unsubscription works.'''
+        """Test that unsubscription works."""
         # pylint: disable-msg=W0212
         # simple one
         o = object()
@@ -105,10 +110,10 @@ class SubscriptionTests(BaseEQTestCase):
 
 
 class PushTests(BaseEQTestCase):
-    '''Test the event distribution machinery.'''
+    """Test the event distribution machinery."""
 
     def test_push_simple(self):
-        '''Test that events can be pushed (not listening yet).'''
+        """Test that events can be pushed (not listening yet)."""
         # not even an event
         self.assertRaises(TypeError, self.eq.push)
 
@@ -129,8 +134,18 @@ class PushTests(BaseEQTestCase):
         self.eq.push("FS_FILE_MOVE", path_from=1, path_to=2)
         self.eq.push("FS_FILE_MOVE", 1, path_to=2)
 
+    def test_events_kwargs(self):
+        """Test that all events are defined correctly with tuples or lists.
+
+        This is to avoid a typical mistake of making it a "(param)", not
+        a "(param,)".
+        """
+        for name, params in event_queue.EVENTS.iteritems():
+            self.assertTrue(isinstance(params, (tuple, list)),
+                            "%s event has params bad defined!" % name)
+
     def test_listened_pushs(self):
-        '''Push events and listem them.'''
+        """Push events and listem them."""
 
         # helper class, pylint: disable-msg=C0111
         class Create(object):
@@ -154,7 +169,7 @@ class PushTests(BaseEQTestCase):
         self.eq.unsubscribe(c)
 
     def test_signatures(self):
-        '''Check that the handle signatures are forced when passing.'''
+        """Check that the handle signatures are forced when passing."""
 
         # helper class, pylint: disable-msg=C0111
         class Create(object):
@@ -185,12 +200,12 @@ class PushTests(BaseEQTestCase):
 
 
 class PushTestsWithCallback(BaseEQTestCase):
-    '''Test the error handling in the event distribution machinery.'''
+    """Test the error handling in the event distribution machinery."""
 
     def test_keep_going(self):
-        ''' Check that if a listener raises an Exception or have a
+        """ Check that if a listener raises an Exception or have a
         wrong signature, the next listeners are called.
-        '''
+        """
         d = defer.Deferred()
         # helper class, pylint: disable-msg=C0111
         class BadListener(object):
@@ -227,7 +242,7 @@ class PushTestsWithCallback(BaseEQTestCase):
         return d
 
     def test_default_handler(self):
-        ''' Check that handler_default is called. '''
+        """ Check that handler_default is called. """
         d = defer.Deferred()
         # helper class, pylint: disable-msg=C0111
         class Listener(object):
@@ -303,41 +318,41 @@ class PushTestsWithCallback(BaseEQTestCase):
 
 
 class MuteFilterTests(unittest.TestCase):
-    '''Tests the MuteFilter class.'''
+    """Tests the MuteFilter class."""
 
     def setUp(self):
         self.mf = event_queue.MuteFilter()
 
     def test_empty(self):
-        '''Nothing there.'''
+        """Nothing there."""
         self.assertFalse(self.mf._cnt)
 
     def test_add_one(self):
-        '''Adds one element.'''
+        """Adds one element."""
         self.mf.add("foo")
         self.assertEqual(self.mf._cnt, dict(foo=1))
 
     def test_add_two_different(self):
-        '''Adds two different elements.'''
+        """Adds two different elements."""
         self.mf.add("foo")
         self.mf.add("bar")
         self.assertEqual(self.mf._cnt, dict(foo=1, bar=1))
 
     def test_add_two_equal(self):
-        '''Adds one element twice.'''
+        """Adds one element twice."""
         self.mf.add("foo")
         self.mf.add("foo")
         self.assertEqual(self.mf._cnt, dict(foo=2))
 
     def test_add_two_equal_and_third(self):
-        '''Adds one element.'''
+        """Adds one element."""
         self.mf.add("foo")
         self.mf.add("bar")
         self.mf.add("bar")
         self.assertEqual(self.mf._cnt, dict(foo=1, bar=2))
 
     def test_pop_simple(self):
-        '''Pops one element.'''
+        """Pops one element."""
         self.mf.add("foo")
         self.assertFalse(self.mf.pop("bar"))
         self.assertEqual(self.mf._cnt, dict(foo=1))
@@ -345,7 +360,7 @@ class MuteFilterTests(unittest.TestCase):
         self.assertFalse(self.mf._cnt)
 
     def test_pop_complex(self):
-        '''Pops several elements.'''
+        """Pops several elements."""
         # add several
         self.mf.add("foo")
         self.mf.add("bar")
@@ -367,6 +382,42 @@ class MuteFilterTests(unittest.TestCase):
         self.assertFalse(self.mf._cnt)
 
 
+class IgnoreFileTests(unittest.TestCase):
+    """Tests the ignore files behaviour."""
+
+    def test_filter_none(self):
+        """Still works ok even if not receiving a regex to ignore."""
+        p = event_queue._GeneralINotifyProcessor(None)
+        self.assertFalse(p.is_ignored("froo.pyc"))
+
+    def test_filter_one(self):
+        """Filters stuff that matches (or not) this one regex."""
+        p = event_queue._GeneralINotifyProcessor(None, ['\A.*\\.pyc\Z'])
+        self.assertTrue(p.is_ignored("froo.pyc"))
+        self.assertFalse(p.is_ignored("froo.pyc.real"))
+        self.assertFalse(p.is_ignored("otherstuff"))
+
+    def test_filter_two_simple(self):
+        """Filters stuff that matches (or not) these simple regexes."""
+        p = event_queue._GeneralINotifyProcessor(None, ['\A.*foo\Z',
+                                                        '\A.*bar\Z'])
+        self.assertTrue(p.is_ignored("blah_foo"))
+        self.assertTrue(p.is_ignored("blah_bar"))
+        self.assertFalse(p.is_ignored("bar_xxx"))
+        self.assertFalse(p.is_ignored("--foo--"))
+        self.assertFalse(p.is_ignored("otherstuff"))
+
+    def test_filter_two_complex(self):
+        """Filters stuff that matches (or not) these complex regexes."""
+        p = event_queue._GeneralINotifyProcessor(None, ['\A.*foo\Z|\Afoo.*\Z',
+                                                        '\A.*bar\Z'])
+        self.assertTrue(p.is_ignored("blah_foo"))
+        self.assertTrue(p.is_ignored("blah_bar"))
+        self.assertTrue(p.is_ignored("foo_xxx"))
+        self.assertFalse(p.is_ignored("--foo--"))
+        self.assertFalse(p.is_ignored("otherstuff"))
+
+
 def test_suite():
     # pylint: disable-msg=C0111
     return unittest.TestLoader().loadTestsFromName(__name__)
