@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-'''Tests the Hashs Queue.'''
+"""Tests the Hashs Queue."""
 
 from __future__ import with_statement
 
@@ -25,6 +25,7 @@ import time
 import logging
 import threading
 
+from StringIO import StringIO
 from twisted.trial.unittest import TestCase as TwistedTestCase
 from twisted.internet import defer, reactor
 
@@ -37,27 +38,27 @@ from ubuntuone.storageprotocol.hash import \
 # this is a unit test we need to access protected members
 # pylint: disable-msg=W0212
 class HasherTests(testcase.BaseTwistedTestCase):
-    '''Test the whole stuff to receive signals.'''
+    """Test the whole stuff to receive signals."""
 
     def setUp(self):
-        '''Setup the test.'''
+        """Setup the test."""
         testcase.BaseTwistedTestCase.setUp(self)
         self.test_dir = self.mktemp('test_dir')
         self.timeout = 2
 
     def tearDown(self):
-        '''Clean up the tests.'''
+        """Clean up the tests."""
         testcase.BaseTwistedTestCase.tearDown(self)
 
     def test_live_process(self):
-        '''Check that the hasher lives and dies.'''
+        """Check that the hasher lives and dies."""
         # create the hasher
         mark = object()
         queue = hash_queue.UniqueQueue()
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, *args):
-                '''callback'''
+                """callback"""
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
         hasher.start()
@@ -71,17 +72,18 @@ class HasherTests(testcase.BaseTwistedTestCase):
 
         # "I see dead threads"
         self.assertFalse(hasher.isAlive())
+        hasher.join(timeout=5)
 
     def test_called_back_ok(self):
-        '''Tests that the hasher produces correct info.'''
+        """Tests that the hasher produces correct info."""
         # create the hasher
         mark = object()
         queue = hash_queue.UniqueQueue()
         d = defer.Deferred()
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, *args):
-                '''callback'''
+                """callback"""
                 d.callback(args)
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
@@ -96,6 +98,8 @@ class HasherTests(testcase.BaseTwistedTestCase):
         def check_info(args):
             """check the info pushed by the hasher"""
             # pylint: disable-msg=W0612
+            hasher.stop()
+            hasher.join(timeout=5)
             event, path, hash, crc, size, stat = args
             self.assertEqual(event, "HQ_HASH_NEW")
             # calculate what we should receive
@@ -109,20 +113,21 @@ class HasherTests(testcase.BaseTwistedTestCase):
                 self.assertEquals(getattr(curr_stat, attr),
                                   getattr(stat, attr))
 
+
         d.addCallback(check_info)
         # release the processor and check
         return d
 
     def test_called_back_error(self):
-        '''Tests that the hasher signals error when no file.'''
+        """Tests that the hasher signals error when no file."""
         # create the hasher
         mark = object()
         queue = hash_queue.UniqueQueue()
         d = defer.Deferred()
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, *args):
-                '''callback'''
+                """callback"""
                 d.callback(args)
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
@@ -133,6 +138,8 @@ class HasherTests(testcase.BaseTwistedTestCase):
 
         def check_info(args):
             """check the info pushed by the hasher"""
+            hasher.stop()
+            hasher.join(timeout=5)
             event, mdid = args
             self.assertEqual(event, "HQ_HASH_ERROR")
             self.assertEqual(mdid, "foo")
@@ -143,7 +150,7 @@ class HasherTests(testcase.BaseTwistedTestCase):
         return d
 
     def test_order(self):
-        '''The hasher should return in order.'''
+        """The hasher should return in order."""
         # calculate what we should receive
         should_be = []
         for i in range(10):
@@ -162,14 +169,16 @@ class HasherTests(testcase.BaseTwistedTestCase):
         d = defer.Deferred()
 
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             # class-closure, cannot use self, pylint: disable-msg=E0213
             def __init__(innerself):
                 innerself.store = []
             def push(innerself, *args):
-                '''callback'''
+                """callback"""
                 innerself.store.append(args)
                 if len(innerself.store) == 10:
+                    hasher.stop()
+                    hasher.join(timeout=5)
                     if innerself.store == should_be:
                         d.callback(True)
                     else:
@@ -186,7 +195,7 @@ class HasherTests(testcase.BaseTwistedTestCase):
         return d
 
     def test_large_content(self):
-        '''The hasher works ok for a lot of info.'''
+        """The hasher works ok for a lot of info."""
         # calculate what we should receive
         testinfo = "".join(chr(random.randint(0, 255)) for i in range(100000))
         hasher = content_hash_factory()
@@ -201,9 +210,11 @@ class HasherTests(testcase.BaseTwistedTestCase):
         d = defer.Deferred()
 
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, event, path, newhash, crc, size, stat):
-                '''callback'''
+                """callback"""
+                hasher.stop()
+                hasher.join(timeout=5)
                 if event != "HQ_HASH_NEW":
                     d.errback(Exception("envent is not HQ_HASH_NEW"))
                 elif path != testfile:
@@ -223,10 +234,10 @@ class HasherTests(testcase.BaseTwistedTestCase):
 
 
 class HashQueueTests(testcase.BaseTwistedTestCase):
-    '''Test the whole stuff to receive signals.'''
+    """Test the whole stuff to receive signals."""
 
     def setUp(self):
-        '''Setup the test.'''
+        """Setup the test."""
         testcase.BaseTwistedTestCase.setUp(self)
         self.test_dir = self.mktemp('test_dir')
         self.timeout = 2
@@ -235,19 +246,19 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
                       self._testMethodName)
 
     def tearDown(self):
-        '''Clean up the tests.'''
+        """Clean up the tests."""
         self.log.info("finished test %s.%s", self.__class__.__name__,
                       self._testMethodName)
         testcase.BaseTwistedTestCase.tearDown(self)
 
     def test_called_back_ok(self):
-        '''Tests that the hasher produces correct info.'''
+        """Tests that the hasher produces correct info."""
         # create the hasher
         d = defer.Deferred()
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, *args):
-                '''callback'''
+                """callback"""
                 d.callback(args)
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
@@ -279,13 +290,13 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         return d
 
     def test_called_back_error(self):
-        '''Tests that the hasher generates an error when no file.'''
+        """Tests that the hasher generates an error when no file."""
         # create the hasher
         d = defer.Deferred()
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, *args):
-                '''callback'''
+                """callback"""
                 d.callback(args)
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
@@ -304,16 +315,16 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         return d
 
     def test_being_hashed(self):
-        '''Tell if something is being hashed.'''
+        """Tell if something is being hashed."""
         tfile1 = os.path.join(self.test_dir, "tfile1")
         open(tfile1, "w").close()
         tfile2 = os.path.join(self.test_dir, "tfile2")
         open(tfile2, "w").close()
 
         class C(object):
-            '''bogus'''
+            """bogus"""
             def push(self, *a):
-                '''none'''
+                """none"""
         hq = hash_queue.HashQueue(C())
         self.addCleanup(hq.shutdown)
 
@@ -343,7 +354,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
 
 
     def test_order(self):
-        '''The hasher should return in order.'''
+        """The hasher should return in order."""
         # calculate what we should receive
         should_be = []
         for i in range(10):
@@ -358,12 +369,12 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
 
         d = defer.Deferred()
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             # class-closure, cannot use self, pylint: disable-msg=E0213
             def __init__(innerself):
                 innerself.store = []
             def push(innerself, *args):
-                '''callback'''
+                """callback"""
                 innerself.store.append(args)
                 if len(innerself.store) == 10:
                     if innerself.store[:-1] == should_be[:-1]:
@@ -382,7 +393,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         return d
 
     def test_unique(self):
-        '''The hasher should return in order.'''
+        """The hasher should return in order."""
         # calculate what we should receive
         should_be = []
         for i in range(10):
@@ -397,12 +408,12 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
 
         d = defer.Deferred()
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             # class-closure, cannot use self, pylint: disable-msg=E0213
             def __init__(innerself):
                 innerself.store = []
             def push(innerself, *args):
-                '''callback'''
+                """callback"""
                 innerself.store.append(args)
                 if len(innerself.store) == 10:
                     if len(innerself.store[:-1]) == len(should_be[:-1]):
@@ -444,7 +455,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
     def test_interrupt_current(self):
         """ Test that the hasher correctly interrupts a inprogress task."""
         # calculate what we should receive
-        testinfo = os.urandom(10000000)
+        testinfo = os.urandom(1000)
         hasher = content_hash_factory()
         hasher.hash_object.update(testinfo)
         testfile = os.path.join(self.test_dir, "testfile")
@@ -456,9 +467,9 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         d = defer.Deferred()
 
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, event, path, newhash, crc, size, stat):
-                '''callback'''
+                """callback"""
                 if event != "HQ_HASH_NEW":
                     d.errback(Exception("envent is not HQ_HASH_NEW"))
                 elif path != testfile:
@@ -467,30 +478,60 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
                     d.errback(Exception("the hashes are different!"))
                 else:
                     d.callback(True)
+
+        class FakeFile(StringIO):
+            """An endless file"""
+            def read(self, size=10):
+                """return random bytes"""
+                return os.urandom(size)
+            # context manager API
+            def __enter__(self):
+                return self
+            def __exit__(self, *args):
+                pass
+
+        # patch __builtin__.open so we don't have to try to interrupt a hashing
+        bi = __import__('__builtin__')
+        old_open = bi.open
+
+        class OpenFaker(object):
+            """A class to fake open for specific paths"""
+            def __init__(self, paths):
+                self.paths = paths
+                self.done = False
+            def __call__(self, path, mode='r'):
+                """the custom open implementation"""
+                if self.done or path not in self.paths:
+                    return old_open(path, mode)
+                else:
+                    self.done = True
+                    return FakeFile()
+        open_faker = OpenFaker([testfile])
+        bi.open = open_faker
+
         receiver = Helper()
-
         hq = hash_queue.HashQueue(receiver)
-        self.addCleanup(hq.shutdown)
-
-        # read in small chunks, so we have more iterations
-        hq.hasher.chunk_size = 2**8
+        def cleanup():
+            """cleanup the mess"""
+            bi.open = old_open
+            hq.shutdown()
+        self.addCleanup(cleanup)
         hq.insert(testfile, "mdid")
+
         # insert it again, to cancel the first one
         reactor.callLater(0.1, hq.insert, testfile, "mdid")
         return d
-    # timeout is used by trial, pylint: disable-msg=W0612
-    test_interrupt_current.timeout = 5
 
     def test_shutdown(self):
         """Test that the HashQueue shutdown """
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, *args):
-                '''callback'''
+                """callback"""
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         hq.shutdown()
-        self.assertRaises(RuntimeError, hq.insert, 'foo', "mdid")
+        self.assertTrue(hq._stopped)
 
     def test_shutdown_while_hashing(self):
         """Test that the HashQueue is shutdown ASAP while it's hashing."""
@@ -499,15 +540,14 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         hasher = content_hash_factory()
         hasher.hash_object.update(testinfo)
         testfile = os.path.join(self.test_dir, "testfile")
-        testhash = hasher.content_hash()
         # send what to hash
         with open(testfile, "w") as fh:
             fh.write(testinfo)
 
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, *args):
-                '''callback'''
+                """callback"""
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         self.addCleanup(hq.shutdown)
@@ -525,15 +565,16 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         self.assertTrue(hq._queue.empty())
 
     def test_insert_post_shutdown(self):
-        '''test inserting a path after the shutdown'''
+        """test inserting a path after the shutdown"""
         class Helper(object):
-            '''helper class'''
+            """helper class"""
             def push(self, *args):
-                '''callback'''
+                """callback"""
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         hq.shutdown()
-        self.assertRaises(RuntimeError, hq.insert, '/foo/bar', "mdid")
+        hq.insert('foo', 'mdid')
+        self.assertFalse(hq.is_hashing('foo', 'mdid'))
 
 
 class UniqueQueueTests(TwistedTestCase):
@@ -561,7 +602,7 @@ class UniqueQueueTests(TwistedTestCase):
         self.assertEquals(0, len(queue._set))
 
     def test_contains(self):
-        '''test contains functionality'''
+        """test contains functionality"""
         queue = hash_queue.UniqueQueue()
 
         # nothing in it
@@ -578,7 +619,7 @@ class UniqueQueueTests(TwistedTestCase):
         self.assertTrue(("item2", "mdid") in queue)
 
     def test_clear(self):
-        '''test clear method'''
+        """test clear method"""
         queue = hash_queue.UniqueQueue()
         queue.put(('item1', "mdid"))
         queue.put(('item2', "mdid"))
@@ -597,7 +638,7 @@ class UniqueQueueTests(TwistedTestCase):
         self.assertEquals(0, queue.qsize())
 
     def test_clear_unfinished_tasks(self):
-        '''test the clear wakeup waiting threads.'''
+        """test the clear wakeup waiting threads."""
         queue = hash_queue.UniqueQueue()
         d = defer.Deferred()
         # helper function, pylint: disable-msg=C0111
