@@ -1,4 +1,3 @@
-#
 # Author: Facundo Batista <facundo@canonical.com>
 # Author: Natalia Bidart <natalia.bidart@canonical.com>
 #
@@ -71,6 +70,7 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
     def setUp(self):
         """ Setup the test """
         testcase.BaseTwistedTestCase.setUp(self)
+
         self.home_dir = self.mktemp('ubuntuonehacker')
         self.shares_dir = self.mktemp('shares')
         usrdir = self.mktemp("usrdir")
@@ -80,7 +80,7 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
         self.fsm = filesystem_manager.FileSystemManager(self.fsmdir,
                                                         self.partials_dir,
                                                         self.vm)
-        self.fsm.create(usrdir, "")
+        self.fsm.create(usrdir, "", is_dir=True)
         self.eq = FakeEQ()
         self.fsm.register_eq(self.eq)
         self.aq = FakeAQ()
@@ -91,18 +91,19 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
 
     @staticmethod
     def create_share(share_id, share_name, fsm, shares_dir,
-                     access_level='Modify'):
+                     access_level='Modify', accepted=True):
         """ creates a share """
         share_path = os.path.join(shares_dir, share_name)
         os.makedirs(share_path)
         share = volume_manager.Share(path=share_path, volume_id=share_id,
-                                     access_level=access_level)
+                                     access_level=access_level,
+                                     accepted=accepted)
         fsm.vm.add_share(share)
         return share
 
     def create_udf(self, udf_id, node_id, suggested_path, path, subscribed):
         """Create an UDF and add it to the volume manager."""
-        volume = volumes.UDFVolume(udf_id, node_id, suggested_path)
+        volume = volumes.UDFVolume(udf_id, node_id, suggested_path.decode('utf-8'))
         udf = volume_manager.UDF.from_udf_volume(volume, path)
         udf.subscribed = subscribed
         self.fsm.vm.add_udf(udf)
@@ -158,7 +159,7 @@ class CollectionTests(BaseTestCase):
     def test_empty_rw(self):
         """Test with one empty Modify share."""
         # create the share
-        share = self.create_share('share_id', 'ro_share',  self.fsm,
+        share = self.create_share('share_id', 'rw_share',  self.fsm,
                                        self.shares_dir, access_level='Modify')
         self.fsm.create(share.path, "share_id", is_dir=True)
         self.fsm.set_node_id(share.path, "uuid")
@@ -374,6 +375,8 @@ class TwistedBase(BaseTestCase):
                                        self.shares_dir, access_level='Modify')
         self.fsm.create(self.share.path, "share_id", is_dir=True)
         self.fsm.set_node_id(self.share.path, "uuidshare")
+        self.share.node_id = "uuidshare"
+        self.vm.shares['share_id'] = self.share
 
     def startTest(self, check_function):
         """Start the test using lr.start()."""
@@ -1850,6 +1853,76 @@ class BadStateTests(TwistedBase):
         return self.deferred
 
 
+class RootBadStateTests(TwistedBase):
+    """Test what happens with volume roots left in a bad state last time."""
+
+    @defer.inlineCallbacks
+    def setUp(self):
+        yield TwistedBase.setUp(self)
+        self.env_var = 'HOME'
+        self.old_value = os.environ.get(self.env_var, None)
+        os.environ[self.env_var] = self.home_dir
+
+
+    def tearDown(self):
+        if self.old_value is None:
+            os.environ.pop(self.env_var)
+        else:
+            os.environ[self.env_var] = self.old_value
+        return TwistedBase.tearDown(self)
+
+    def _test_it(self, volume):
+        """Run the bad state test for a specific volume."""
+        path = volume.path
+        mdid = self.fsm.get_by_path(path).mdid
+        partial_path = os.path.join(self.fsm.partials_dir,
+                                mdid + ".u1partial." + os.path.basename(path))
+
+        # start the download, never complete it
+        self.fsm.set_by_mdid(mdid, server_hash="blah-hash-blah")
+        self.fsm.create_partial(volume.node_id, volume.volume_id)
+        fh = self.fsm.get_partial_for_writing(volume.node_id, volume.volume_id)
+        fh.write("foobar")
+        self.assertTrue(os.path.exists(partial_path))
+
+        def check(_):
+            """arrange the metadata so later server_rescan will do ok"""
+            mdobj = self.fsm.get_by_mdid(mdid)
+            self.assertFalse(mdobj.info.is_partial)
+            self.assertEqual(mdobj.server_hash, mdobj.local_hash)
+            self.assertFalse(os.path.exists(partial_path))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_SERVER_root(self):
+        """We were downloading the root dir, but it was interrupted."""
+        # create the root
+        self.fsm.set_node_id(self.vm.root.path, self.vm.root.node_id)
+        return self._test_it(self.vm.root)
+
+    def test_SERVER_share(self):
+        """We were downloading the share root dir but it was interrupted."""
+        # create a share
+        share = self.create_share('share_id_1', 'rw_share',  self.fsm,
+                                       self.shares_dir, access_level='Modify')
+        self.fsm.create(share.path, "share_id_1", is_dir=True)
+        self.fsm.set_node_id(share.path, "uuid_share_1")
+        share.node_id = "uuid_share_1"
+        self.vm.shares['share_id_1'] = share
+        return self._test_it(share)
+
+    def test_SERVER_udf(self):
+        """We were downloading the udf root dir, but it was interrupted."""
+        udf_path = os.path.join(self.home_dir, "myudf")
+        os.mkdir(udf_path)
+        udf = self.create_udf('udf_id', 'udf_root_node_id', "~/myudf",
+                                   udf_path, True)
+        self.fsm.create(udf.path, 'udf_id', is_dir=True)
+        self.fsm.set_node_id(udf.path, 'udf_root_node_id')
+        return self._test_it(udf)
+
+
 class TrashTests(TwistedBase):
     """Test handling trash."""
 
