@@ -30,11 +30,13 @@ from dbus.mainloop.glib import DBusGMainLoop
 from contrib.testing.testcase import (
     FakeVolumeManager,
     BaseTwistedTestCase,
-    MementoHandler, DummyClass
+    MementoHandler
 )
 
+from contrib.testing import testcase
 from ubuntuone.syncdaemon.dbus_interface import DBusInterface
 from ubuntuone.syncdaemon.filesystem_manager import FileSystemManager
+from ubuntuone.syncdaemon.fsm import fsm as fsm_module
 from ubuntuone.syncdaemon.main import Main
 from ubuntuone.syncdaemon.sync import FSKey, Sync, SyncStateMachineRunner
 from ubuntuone.syncdaemon.volume_manager import Share
@@ -180,8 +182,8 @@ class FSKeyTests(FSKeyTestCase):
                              self.fsm.get_by_path(path).server_hash)
 
 
-class TestSync(BaseTwistedTestCase):
-    """Test for Sync."""
+class BaseSync(BaseTwistedTestCase):
+    """Base test infrastructure for Sync."""
 
     def setUp(self):
         """Init."""
@@ -218,6 +220,10 @@ class TestSync(BaseTwistedTestCase):
                 raise exc_info[0], exc_info[1], exc_info[2]
         BaseTwistedTestCase.tearDown(self)
 
+
+class TestSync(BaseSync):
+    """Test for Sync."""
+
     def test_deleting_open_files_is_no_cause_for_despair(self):
         """test_deleting_open_files_is_no_cause_for_despair."""
         def cb(_):
@@ -256,30 +262,157 @@ class TestSync(BaseTwistedTestCase):
         def faked_nothing(ssmr, event, params, *args):
             """Wrap SSMR.nothing to test."""
             self.called = True
+        self.patch(SyncStateMachineRunner, 'nothing', faked_nothing)
 
-        SyncStateMachineRunner.nothing = faked_nothing
         kwargs = dict(share_id='share_id', node_id='node_id')
         sync.handle_AQ_DOWNLOAD_DOES_NOT_EXIST(**kwargs)
         self.assertTrue(self.called, 'nothing was called')
 
+    def test_handle_FILE_CREATE_while_LOCAL(self):
+        """A FS_FILE_CREATE is received with the node in LOCAL."""
+        sync = Sync(main=self.main)
+        self.called = False
 
-class SyncStateMachineRunnerTestCase(BaseTwistedTestCase):
+        def faked_nothing(ssmr, event, params, *args):
+            """Wrap SSMR.nothing to test."""
+            self.called = True
+        self.patch(SyncStateMachineRunner, 'nothing', faked_nothing)
+
+        # create a file and put it in local
+        fsm = self.main.fs
+        somepath = os.path.join(self.root, 'somepath')
+        mdid = fsm.create(somepath, '')
+        fsm.set_by_mdid(mdid, local_hash='somehash')
+
+        # send the event, and check that it called its .nothing()
+        sync.handle_FS_FILE_CREATE(somepath)
+        self.assertTrue(self.called)
+
+    def test_SV_HASH_NEW_with_file_uploadinterrupted(self):
+        """A SV_HASH_NEW is received after upload interrupted."""
+        sync = Sync(main=self.main)
+        self.called = False
+
+        def fake_meth(_, event, params, hash):
+            """Wrap SSMR.reput_file_from_local to test."""
+            self.assertEqual(event, 'SV_HASH_NEW')
+            self.assertEqual(hash, '')
+            self.called = True
+        self.patch(SyncStateMachineRunner, 'reput_file_from_local', fake_meth)
+
+        # create a file and put it in local, without server_hash, as
+        # if the upload was cut in the middle after the make file
+        fsm = self.main.fs
+        somepath = os.path.join(self.root, 'somepath')
+        mdid = fsm.create(somepath, '', node_id='node_id')
+        fsm.set_by_mdid(mdid, local_hash='somehash', crc32='crc32',
+                        stat='stat', size='size')
+
+        # send the event and check
+        mdobj = fsm.get_by_mdid(mdid)
+        sync.handle_SV_HASH_NEW(mdobj.share_id, mdobj.node_id, '') # no content
+        self.assertTrue(self.called)
+
+
+class SyncStateMachineRunnerTestCase(BaseSync):
     """Tests for the SyncStateMachineRunner."""
 
     def setUp(self):
         """Init."""
-        BaseTwistedTestCase.setUp(self)
-        self.ssmr = SyncStateMachineRunner(fsm=None, main=None,
-                                           key=DummyClass(), logger=None)
+        BaseSync.setUp(self)
+
+        # create a file
+        self.fsm = self.main.fs
+        somepath = os.path.join(self.root, 'somepath')
+        self.mdid = self.fsm.create(somepath, '', node_id='node_id')
+
+        key = FSKey(self.main.fs, share_id='', node_id='node_id')
+        self.ssmr = SyncStateMachineRunner(fsm=self.main.fs, main=self.main,
+                                           key=key, logger=None)
 
     def tearDown(self):
         """Clean up."""
         self.ssmr = None
-        BaseTwistedTestCase.tearDown(self)
+        BaseSync.tearDown(self)
 
-    def test_delete_file(self):
-        """delete_file can be called with or without the server hash."""
+    def test_delete_file_without_hash(self):
+        """Delete_file can be called without the server hash."""
         self.ssmr.delete_file(event='AQ_DOWNLOAD_ERROR', params=None)
 
+    def test_delete_file_with_hash(self):
+        """Delete_file can be called with the server hash."""
         self.ssmr.delete_file(event='AQ_DOWNLOAD_ERROR', params=None,
                               server_hash='')
+
+    def test_put_file_stores_info(self):
+        """The put_file method should store the info in FSM."""
+        self.ssmr.put_file('HQ_HASH_NEW', None, 'hash', 'crc', 'size', 'stt')
+
+        # check the info is stored
+        mdobj = self.fsm.get_by_mdid(self.mdid)
+        self.assertEqual(mdobj.local_hash, 'hash')
+        self.assertEqual(mdobj.crc32, 'crc')
+        self.assertEqual(mdobj.size, 'size')
+        self.assertEqual(mdobj.stat, 'stt')
+
+    def test_reput_file_stores_info(self):
+        """The reput_file method should store the info in FSM."""
+        self.ssmr.reput_file('HQ_HASH_NEW', None, 'hash', 'crc', 'size', 'stt')
+
+        # check the info is stored
+        mdobj = self.fsm.get_by_mdid(self.mdid)
+        self.assertEqual(mdobj.local_hash, 'hash')
+        self.assertEqual(mdobj.crc32, 'crc')
+        self.assertEqual(mdobj.size, 'size')
+        self.assertEqual(mdobj.stat, 'stt')
+
+
+class FakedState(object):
+    """A faked state."""
+
+    def __init__(self, action_func):
+        self.action_func = action_func
+        self.values = []
+
+    def get_transition(self, event_name, parameters):
+        """A fake get_transition."""
+        class A(object): pass
+        result = A()
+        result.action_func = self.action_func
+        result.target = self.values
+        return result
+
+
+class StateMachineRunnerTestCase(BaseTwistedTestCase):
+    """Test suite for StateMachineRunner."""
+
+    def setUp(self):
+        """Init."""
+        BaseTwistedTestCase.setUp(self)
+
+        logger = logging.getLogger('ubuntuone.SyncDaemon.sync')
+        self.smr = fsm_module.StateMachineRunner(None, logger)
+        self.handler = testcase.MementoHandler()
+        self.handler.setLevel(logging.INFO)
+        logger.addHandler(self.handler)
+
+    def test_on_event_logs_info(self):
+        """Test proper logging in on_event."""
+        event_name = 'TEST_EVENT'
+        parameters = object()
+        action_func = 'SOME_ACTION_FUNC'
+        setattr(self.smr, action_func, lambda *_: None)
+        self.patch(self.smr, 'get_state', lambda: FakedState(action_func))
+        self.smr.on_event(event_name=event_name, parameters=parameters)
+
+        # assert over logging
+        self.assertTrue(len(self.handler.records), 1)
+
+        record = self.handler.records[0]
+        self.assertEqual(record.levelno, logging.INFO)
+
+        msg = record.message
+        error = '%s must be in record.message (%s)'
+        self.assertTrue(event_name in msg, error % (event_name, msg))
+        self.assertTrue(str(parameters) in msg, error % (parameters, msg))
+        self.assertTrue(action_func in msg, error % (action_func, msg))
