@@ -380,7 +380,7 @@ class TestToolsBasic(TestToolsBase):
     @defer.inlineCallbacks
     def test_delete_folder(self):
         """Test for Folders.delete."""
-        path = os.path.join(self.home_dir, 'ñoño')
+        path = os.path.join(self.home_dir, u'ñoño').encode('utf-8')
         # patch AQ.delete_volume
         def delete_volume(volume_id):
             """Fake delete_volume"""
@@ -388,7 +388,7 @@ class TestToolsBasic(TestToolsBase):
         self.main.action_q.delete_volume = delete_volume
 
         # create sample folder
-        suggested_path = os.path.join("~", 'ñoño')
+        suggested_path = os.path.join("~", u'ñoño').decode('utf-8')
         udf = volume_manager.UDF("folder_id", "node_id", suggested_path, path,
                                  subscribed=True)
         yield self.main.vm.add_udf(udf)
