@@ -301,8 +301,8 @@ class U1SDToolTests(TestToolsBase):
     def test_show_folders(self):
         """test the output of --list-folders """
         out = StringIO()
-        path = 'ñoño'
-        suggested_path = os.path.join("~", 'ñoño')
+        path = u'ñoño'.encode('utf-8')
+        suggested_path = os.path.join("~", u'ñoño')
 
         udf = UDF("folder_id", "node_id", suggested_path, path,
                   subscribed=True)
