@@ -32,7 +32,7 @@ from twisted.python.failure import Failure
 from ubuntuone.syncdaemon.event_queue import EVENTS
 from ubuntuone.syncdaemon.interfaces import IMarker
 from ubuntuone.syncdaemon import config
-from ubuntuone.syncdaemon.volume_manager import Share, UDF
+from ubuntuone.syncdaemon.volume_manager import Share, UDF, VolumeDoesNotExist
 
 
 # Disable the "Invalid Name" check here, as we have lots of DBus style names
@@ -669,9 +669,9 @@ class EventListener(object):
         self.handle_default('VM_VOLUME_DELETE_ERROR', volume_id, error)
         try:
             volume = self.dbus_iface.volume_manager.get_volume(volume_id)
-        except KeyError:
+        except VolumeDoesNotExist:
             logger.error("Unable to handle VM_VOLUME_DELETE_ERROR for "
-                         "volume_id=%r", volume_id)
+                         "volume_id=%r, no such volume.", volume_id)
         else:
             if isinstance(volume, Share):
                 self.dbus_iface.shares.emit_share_delete_error(volume, error)
@@ -886,10 +886,12 @@ class Shares(DBusExposedObject):
         try:
             self.vm.delete_volume(str(share_id))
             reply_handler()
-        except KeyError:
-            error_handler(ValueError("The volume with id: %s don't exists" % \
-                                     str(share_id)))
-
+        except VolumeDoesNotExist, e:
+            self.ShareDeleteError({'volume_id':share_id}, str(e))
+        except Exception, e:
+            error_handler(e)
+            logger.exception('Error while deleting share: %r', share_id)
+            self.ShareDeleteError({'volume_id':share_id}, str(e))
 
     @dbus.service.signal(DBUS_IFACE_SHARES_NAME,
                          signature='a{ss}')
@@ -1214,7 +1216,7 @@ class Config(DBusExposedObject):
         """Enable UDF autosubscribe."""
         logger.debug('called set_files_sync_enabled %d', enabled)
         user_config = config.get_user_config()
-        user_config.set_files_sync_enabled(enabled)
+        user_config.set_files_sync_enabled(bool(int(enabled)))
         user_config.save()
 
     @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
@@ -1246,6 +1248,8 @@ class Folders(DBusExposedObject):
                 udf_dict[unicode(k)] = self.bool_str(v)
             elif k == 'path':
                 udf_dict[unicode(k)] = v.decode('utf-8')
+            elif k == 'suggested_path' and isinstance(v, str):
+                udf_dict[unicode(k)] = v.decode('utf-8')
             else:
                 udf_dict[unicode(k)] = unicode(v)
         return udf_dict
@@ -1267,9 +1271,11 @@ class Folders(DBusExposedObject):
         logger.debug('Folders.delete: %r', folder_id)
         try:
             self.vm.delete_volume(folder_id)
+        except VolumeDoesNotExist, e:
+            self.FolderDeleteError({'volume_id':folder_id}, str(e))
         except Exception, e:
             logger.exception('Error while deleting volume: %r', folder_id)
-            self.emit_folder_delete_error({'id':folder_id}, str(e))
+            self.FolderDeleteError({'volume_id':folder_id}, str(e))
 
     @dbus.service.method(DBUS_IFACE_FOLDERS_NAME, out_signature='aa{ss}')
     def get_folders(self):
