@@ -577,7 +577,7 @@ class _GeneralINotifyProcessor(pyinotify.ProcessEvent):
         return False
 
     def handle_dir_delete(self, fullpath):
-        """ handle the case of move a dir to a non-watched directory """
+        """Handle the case of move a dir to a non-watched directory."""
         paths = self.eq.fs.get_paths_starting_with(fullpath,
                                                    include_base=False)
         paths.sort(reverse=True)
@@ -739,8 +739,13 @@ class EventQueue(object):
         The signature for each event is forced on each method, not in this
         'push' arguments.
         """
-        self.log.debug("push_event: %s, args:%s, kw:%s",
-                     event_name, args, kwargs)
+        log_msg = "push_event: %s, args:%s, kw:%s"
+        if event_name.endswith('DELETE'):
+            # log every DELETE in INFO level
+            self.log.info(log_msg, event_name, args, kwargs)
+        else:
+            self.log.debug(log_msg, event_name, args, kwargs)
+
         # get the event parameters
         try:
             event_params = EVENTS[event_name]
