@@ -29,6 +29,7 @@ import errno
 import stat
 
 from ubuntuone.syncdaemon import file_shelf
+from ubuntuone.syncdaemon.volume_manager import VolumeDoesNotExist
 import uuid
 
 METADATA_VERSION = "4"
@@ -277,7 +278,7 @@ class FileSystemManager(object):
                 # check if the share exists
                 try:
                     self._get_share(mdobj["share_id"])
-                except KeyError:
+                except VolumeDoesNotExist:
                     # oops, the share is gone!, invalidate this mdid
                     log_warning('Share %r disappeared! deleting mdid: %s', mdobj['share_id'], mdid)
                     del self.fs[mdid]
