@@ -131,7 +131,8 @@ class StateMachineRunner(object):
             self.log.debug("passing")
         else:
             # pylint: disable-msg=W0703
-            self.log.debug("Calling %s"%action_func_name)
+            self.log.info("Calling %s (got %s:%s)",
+                          action_func_name, event_name, parameters)
             try:
                 af(event_name, parameters, *args)
             except Exception, e:
@@ -158,7 +159,7 @@ class StateMachineRunner(object):
                 enter_state.values, event_name, parameters,
                 out_state.values, transition.target))
             raise ValueError("Incorrect out state")
-        self.log.debug("Called %s"%action_func_name)
+        self.log.debug("Called %s", action_func_name)
         return action_func_name
 
     def get_state(self):
@@ -193,8 +194,7 @@ class StateMachine(object):
                 # fsm_parser depends on python-uno for reading ods documents
                 # this shouldnt be called with an .ods file on production
                 # environments
-                from ubuntuone.syncdaemon.fsm import \
-                    fsm_parser
+                from ubuntuone.syncdaemon.fsm import fsm_parser
                 spec = fsm_parser.parse(input_data)
             elif input_data.endswith(".py"):
                 result = {}
