@@ -302,6 +302,25 @@ class LocalRescan(object):
         # check all directories
         to_scan_later = []
         events = []
+
+        # check if dirpath is the share root
+        if dirpath == share.path:
+            fullname = share.path
+            mdobj = self.fsm.get_by_path(fullname)
+            changed = self.fsm.changed(mdid=mdobj.mdid)
+            if changed == "SERVER":
+                # download interrupted
+                log_debug("checking root: %r in SERVER, fixing hash and "
+                          "removing partial.", fullname)
+                self.fsm.set_by_mdid(mdobj.mdid,
+                                     server_hash=mdobj.local_hash)
+                self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
+            elif changed == "NONE":
+                log_debug("checking root: %r in NONE, ok!", fullname)
+            else:
+                log_warning("checking root: %r in wrong changed "
+                            "value '%s'", fullname, changed)
+
         for dname in dirnames:
             fullname = os.path.join(dirpath, dname)
             if dname in shouldbe:
