@@ -669,10 +669,22 @@ class SyncStateMachineRunner(StateMachineRunner):
         """Starts the scan again on a dir."""
         self.m.lr.scan_dir(self.key['mdid'], self.key['path'], udfmode)
 
-    def put_file(self, event, params, hash, crc32, size, stat):
-        """upload the file to the server."""
+    def reput_file_from_local(self, event, params, hash):
+        """Re put the file from its local state."""
+        local_hash = self.key['local_hash']
         previous_hash = self.key['server_hash']
-        self.key.set(local_hash=hash, stat=stat)
+        crc32 = self.key['crc32']
+        size = self.key['size']
+        share_id = self.key['share_id']
+        node_id = self.key['node_id']
+
+        self.m.action_q.upload(share_id, node_id, previous_hash, local_hash,
+                               crc32, size, self.key.open_file)
+
+    def put_file(self, event, params, hash, crc32, size, stat):
+        """Upload the file to the server."""
+        previous_hash = self.key['server_hash']
+        self.key.set(local_hash=hash, stat=stat, crc32=crc32, size=size)
         self.key.sync()
 
         self.m.action_q.upload(share_id=self.key['share_id'],
@@ -699,12 +711,12 @@ class SyncStateMachineRunner(StateMachineRunner):
 
 
     def reput_file(self, event, param, hash, crc32, size, stat):
-        """put the file again."""
+        """Put the file again."""
         self.m.action_q.cancel_upload(share_id=self.key['share_id'],
                             node_id=self.key['node_id'])
         previous_hash = self.key['server_hash']
 
-        self.key.set(local_hash=hash, stat=stat)
+        self.key.set(local_hash=hash, stat=stat, crc32=crc32, size=size)
         self.key.sync()
         self.m.action_q.upload(share_id=self.key['share_id'],
             node_id=self.key['node_id'], previous_hash=previous_hash,
