@@ -276,6 +276,20 @@ class UDF(Volume):
         return result
 
 
+class VolumeDoesNotExist(Exception):
+    """Exception for non existing volumes."""
+
+    msg = 'DOES_NOT_EXIST'
+
+    def __init__(self, volume_id):
+        """Create the instance."""
+        super(VolumeDoesNotExist, self).__init__(self.msg, volume_id)
+
+    def __str__(self):
+        """The error message."""
+        return self.msg
+
+
 class VolumeManager(object):
     """Manages shares and mount points."""
 
@@ -309,6 +323,10 @@ class VolumeManager(object):
         if not os.path.exists(self.m.shares_dir_link):
             self.log.debug('creating Shares symlink: %r -> %r',
                            self.m.shares_dir_link, self.m.shares_dir)
+            # remove the symlink if it's broken
+            if os.path.islink(self.m.shares_dir_link) and \
+               os.readlink(self.m.shares_dir_link) != self.m.shares_dir:
+                os.remove(self.m.shares_dir_link)
             os.symlink(self.m.shares_dir, self.m.shares_dir_link)
         # make the shares_dir read only
         os.chmod(self.m.shares_dir, 0555)
@@ -803,13 +821,13 @@ class VolumeManager(object):
         del self.udfs[udf_id]
         self.m.event_q.push('VM_VOLUME_DELETED', udf)
 
-    def get_volume(self, id):
+    def get_volume(self, volume_id):
         """Returns the Share or UDF with the matching id."""
-        volume = self.shares.get(id, None)
+        volume = self.shares.get(volume_id, None)
         if volume is None:
-            volume = self.udfs.get(id, None)
+            volume = self.udfs.get(volume_id, None)
         if volume is None:
-            raise KeyError(id)
+            raise VolumeDoesNotExist(volume_id)
         return volume
 
     def get_volumes(self, all_volumes=False):
@@ -868,10 +886,13 @@ class VolumeManager(object):
                     return
                 # the UDF it's in subscribed state by default
                 subscribe = config.get_user_config().get_udf_autosubscribe()
-                udf = UDF(None, None, os.path.join(udf_path, udf_name), path,
-                          subscribed=subscribe)
+                udf = UDF(None, None,
+                         # suggested_path is always unicode
+                          os.path.join(udf_path, udf_name).decode('utf-8'),
+                          path, subscribed=subscribe)
                 self.marker_udf_map[marker] = udf
-                self.m.action_q.create_udf(udf_path, udf_name, marker)
+                self.m.action_q.create_udf(udf_path.decode('utf-8'),
+                                           udf_name.decode('utf-8'), marker)
             except Exception, e:
                 self.m.event_q.push('VM_UDF_CREATE_ERROR', path,
                                     "UNKNOWN_ERROR: %s" % e)
@@ -913,13 +934,8 @@ class VolumeManager(object):
 
         """
         self.log.info('delete_volume: %r', volume_id)
-        try:
-            volume = self.get_volume(volume_id)
-        except KeyError:
-            self.m.event_q.push('VM_VOLUME_DELETE_ERROR', volume_id,
-                                "DOES_NOT_EXIST")
-        else:
-            self.m.action_q.delete_volume(volume.id)
+        volume = self.get_volume(volume_id)
+        self.m.action_q.delete_volume(volume.id)
 
     def subscribe_udf(self, udf_id):
         """Mark the UDF with id as subscribed.
